function [frequencies directions differenceCode startingLocations] = adaptive_chain_code(inputImage, isUniform)
% ADAPTIVE_CHAIN_CODE Adaptive Chain Code for Boundaries in Images
%   [frequencies directions differenceCode] = adaptive_chain_code(inputImage)
%
%   inputImage is a binary image array object map, where a pixel is 1 if
%     the pixel belongs to the object, and 0 otherwise
%   
%   frequencies is a n*8 array indicating the histogram of each possible 8 changes in angle
%   (0, +45, +90, +135, 180, -135, -90, -45), where n is the number of
%   detected boundary curves.
%
%   directions is an (n x 1) cell array containing the absolute directions that each
%   boundary curve traces on the image
%  
%   differenceCode is an (n x 1) cell array of changes in angle of the
%   chain codes
%
%Authors: Hossein Mobahi, Shankar Rao
%Contact: Shankar Rao <srrao@illinois.edu>
%(c) Copyright. University of Illinois at Urbana-Champaign. 2008.
%               University of California, Berkeley. 2008.
%
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of their author(s). The authors are not responsible
%for any potential property loss or damage caused directly or indirectly by
%the usage of the software.
%

if nargin < 2,
    isUniform = false;
end

if isUniform,
    onBoundary = false(size(inputImage));
    onBoundary(2:end,:) = onBoundary(2:end,:) | (inputImage(2:end,:) & ~inputImage(1:end-1,:));
    onBoundary(1:end-1,:) = onBoundary(1:end-1,:) | (inputImage(1:end-1,:) & ~inputImage(2:end,:));
    onBoundary(:,2:end) = onBoundary(:,2:end) | (inputImage(:,2:end) & ~inputImage(:,1:end-1));
    onBoundary(:,1:end-1) = onBoundary(:,1:end-1) | (inputImage(:,1:end-1) & ~inputImage(:,2:end));
    [boundaryLabels boundaryCount] = bwlabel(onBoundary,4);
    frequencies = zeros(boundaryCount, 8);
    for boundaryIndex = 1:boundaryCount
        frequencies(boundaryIndex,1) = sum(sum(boundaryLabels==boundaryIndex));
    end
    directions = [];
    differenceCode = [];
    return;
end

codebook = [3 2 1 4 -1 0 5 6 7];

[height width] = size(inputImage);
boundary = bwboundaries(inputImage);
chainCount = length(boundary);

% bwboundaries considers pixels at the edge of the image as boundary pixels
% when we do not, so we must remove these pixels and potentially break up
% the curves into multiple segments.
newBoundary = {};
for chainIndex = 1:chainCount
    % b = current boundary curve we are traversing
    b = boundary{chainIndex};
    % i = index along the current boundary curve
    state = false;
    for i=1:length(b)
        isRealBoundaryPixel = ~((b(i,1)==1) || (b(i,1)==height) || (b(i,2)==1) || (b(i,2)==width));
        if ~isRealBoundaryPixel && state
            newBoundary = [ newBoundary b(start:i-1,:) ];
            state = false;
        end
        if isRealBoundaryPixel && ~state
           start = i;
           state = true;
        end
    end
    if state
        newBoundary = [ newBoundary b(start:i,:) ];
    end
end
boundary = newBoundary;
chainCount = length(boundary);

directions = cell(chainCount,1);
differenceCode = cell(chainCount,1);
frequencies = zeros(chainCount, 8);
if nargout == 4,
    startingLocations = zeros(chainCount,2);
end

for chainIndex = 1:chainCount
    % directionVectors should contain a set of 2-vectors in base 3. 
    % [0 0]  [0 1]  [0 2]
    %      \   |   /
    %       \  |  /
    %        \ | /
    %         \|/
    % [1 0]----*----[1 2]
    %         /|\
    %        / | \
    %       /  |  \
    %      /   |   \
    % [2 0]  [2 1]  [2 2]
    directionVectors = diff(boundary{chainIndex}, 1, 1)+1;
    if nargout == 4,
        startingLocations(chainIndex,:) = boundary{chainIndex}(1,:);
    end
    % Embed the directionVectors so that each direction is mapped to an
    % integer between 0 and 7 as follows:
    % 3  2  1
    %  \ | /
    %   \|/
    % 4--*--0
    %   /|\
    %  / | \
    % 5  6  7
    directions{chainIndex} = codebook(directionVectors(:,1)*3 + directionVectors(:,2)+1);
    differenceCode{chainIndex} = mod(diff(directions{chainIndex}, 1, 2), 8);
    if ~isempty(differenceCode{chainIndex})    
        frequencies(chainIndex, :) = histc(differenceCode{chainIndex}, 0:7);
    end
end