% CODING_LENGTH_IMAGE coding length function specialized for image
% segmentation.
%
% [codingLength, codingLengthData, codingLengthGroup] = coding_length_image( ...
%    region, epsilon2, imageSize, windowIndex, windowSize, parameters)
%
%  region is a structure containing the parameters of the region.
%  It contains the fields 'label', 'center', 'covariance', 
%  'interiorPixelCounts', 'totalPixelCount', and 'chainCodeCounts'. 
%
%   epsilon2 is the square of the noise tolerance
%
%   imageSize is the dimensions of the image. only used if useChainCode is
%   false.
%
%   windowIndex is the index of the window size in the array of window
%   size. This is needed to interact with the region structure.
%
%   windowSize is the number pixel along one edge of a window. only used if
%   overlappingWindows is true
%
%   parameters is a structure containing the coding parameters. It includes
%   the fields 'useChainCode', 'overlappingWindows', and 'chainCodeBitCounts'
%
%   - useChainCode is true we are using the chain code to model the
%   boundary of the image and false if we use the huffman code to model
%   group membership
%
%   - overlappingWindows is true if we model the windows associated to every
%   pixel in the region, or false if model only enough windows to represent 
%   the region in the image.
%
%   - chainCodeBitCounts is a (1 x 8) matrix containing the apriori frequency
%   of each chain code. Only used if useChainCode is true
%
%   codingLength is the computed coding length of the region subject to
%   distortion epsilon^2
%
%   codingLengthData is the number of bits needed to represent the mean and
%   covariance of the data
%
%   codingLengthGroup is the number of bits needed to represent the group
%   membership of the data (using either a Huffman or adaptive chain code).
%
%Authors: Shankar Rao, Hossein Mobahi
%Contact: Shankar Rao <srrao@illinois.edu>
%
%(c) Copyright. University of Illinois at Urbana-Champaign. 
%               University of California, Berkeley. 2009.
%
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of their author(s). The authors are not responsible
%for any potential property loss or damage caused directly or indirectly by
%the usage of the software.

function [codingLength, codingLengthData, codingLengthGroup] = coding_length_image( ...
    region, epsilon2, imageSize, windowIndex, windowSize, parameters)

if nargin == 0,
    codingLength.affineModel = true;
    codingLength.useChainCode = true;
    codingLength.overlappingWindows = false;

    %codingLength.useEdgePenalty = false;
    %codingLength.avoidStrongEdges = false;
    %codingLength.strongEdgeThreshold = 8;
    %codingLength.edgeAlpha = 1;
    %codingLength.edgeDelta = 3;
    codingLength.chainCodeMode = 'adaptive';

    return;
end

if strcmp(parameters.chainCodeMode, 'adaptive'),
    chainCodeBitCounts = [ ...
        0.584800772633366 ...
        0.189661746246865 ...
        0.020453687819463 ...
        0 ...
        0.002104190215308 ...
        0.002890183173518 ...
        0.030969517355864 ...
        0.169119902555616 ];
else
    chainCodeBitCounts = ones(1,8)/8;
end

mu = region.center{windowIndex};
Sigma = region.covariance{windowIndex};


dimensionCount = size(mu,1);


if ~parameters.overlappingWindows
    windowCount = region.totalPixelCount / (windowSize^2);
    totalSampleCount = prod(ceil(imageSize/windowSize));
else
    windowCount = region.totalPixelCount;
    totalSampleCount = prod(imageSize);
end


codingLengthData = coding_length_mean_cov(Sigma, windowCount, dimensionCount, epsilon2, parameters.affineModel, mu);


if parameters.useChainCode
    pixelBitCount = sum(ceil(log2(imageSize)));
    codingLengthGroup = coding_length_chain(region.chainCodeCounts, ...
        chainCodeBitCounts, pixelBitCount);
else
    codingLengthGroup = coding_length_huffman(windowCount, totalSampleCount);
end

codingLength = codingLengthData + codingLengthGroup;
