% CONSTRUCT_JOINT_REGION_PARAMETERS compute the parameters a region formed
% by merging together two other regions.
%
% jointRegion = combine_joint_region_parameters(...
%    region1, region2, pixelLabels, pixelFeatures, windowIndices, windowSizeArray)
%
%   region1 is a struct containing the parameters of the first region.
%
%   region2 is a struct containing the parameters of the second region.
%
%   pixelLabels is an array the size of the image that contains the label
%   of every pixel (which region each pixel belongs to)
%
%   pixelFeatures is a (d x N) array where each column contains the
%   features associated a pixel in the image, where d is the dimension of
%   feature vector and N is the total number of pixels in the image. The
%   pixels are indexed linearly (as opposed to being indexed by x,y
%   coordinates) in this matrix for speed.
%
%   windowIndices is an array containing the indices of window sizes against 
%   which we will construct the region parameters 
%
%   windowSizeArray is the set of windows sizes around each pixel to be used.
%   This must consist of odd integers, be strictly increasing, and contain
%   the number 1 (because no window of size 1 can be degenerate).
%
%   jointRegion is a struct containing the parameters of the joint region.
%
%Authors: Shankar Rao
%Contact: Shankar Rao <srrao@illinois.edu>
%
%(c) Copyright. University of Illinois at Urbana-Champaign.
%               University of California, Berkeley. 2009.
%
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of their author(s). The authors are not responsible
%for any potential property loss or damage caused directly or indirectly by
%the usage of the software.

function jointRegion = combine_joint_region_parameters(...
    region1, region2, pixelLabels, pixelFeatures, windowIndices, windowSizeArray, useUniformCode)

if nargin < 7 
    useUniformCode = false;
end

windowSizeCount = length(windowSizeArray);
inJointRegion = pixelLabels == region1.label | pixelLabels == region2.label;

    jointRegion = struct('label', min(region1.label, region2.label), ...
        'center', {cell(windowSizeCount,1)}, ...
        'covariance', {cell(windowSizeCount,1)}, ...
        'interiorPixelCounts', zeros(1,windowSizeCount), ...
        'totalPixelCount', region1.totalPixelCount+region2.totalPixelCount, ...
        'chainCodeCounts', adaptive_chain_code(inJointRegion, useUniformCode));




for windowIndex = windowIndices,
    windowSize = windowSizeArray(windowIndex);
    mu1 = region1.center{windowIndex};
    Sigma1 = region1.covariance{windowIndex};
    n1 = region1.interiorPixelCounts(windowIndex);
    mu2 = region2.center{windowIndex};
    Sigma2 = region2.covariance{windowIndex};
    n2 = region2.interiorPixelCounts(windowIndex);
    [mu_hat, Sigma_hat, n_hat] = combine_mean_cov(mu1, Sigma1, n1, mu2, Sigma2, n2);

    [xIndices, yIndices] = doubleregion(pixelLabels, region1.label, region2.label, windowSize);

    borderIndices = sub2ind(size(pixelLabels), xIndices, yIndices);
    
    if ~isempty(borderIndices)
        [borderMu, borderSigma, border_n] = construct_mean_cov(pixelFeatures{windowIndex}, borderIndices);
        [mu_hat, Sigma_hat, n_hat] = combine_mean_cov(mu_hat, Sigma_hat, n_hat, borderMu, borderSigma, border_n);
    end
        
    
    jointRegion.center{windowIndex} = mu_hat;
    jointRegion.covariance{windowIndex} = Sigma_hat;
    if ~isempty(borderIndices) || windowSize == 1,
        jointRegion.interiorPixelCounts(windowIndex) = n_hat;
    elseif windowSize ~=1,
        % signal that merging these two regions would produce no new
        % interior windows
        %fprintf('Merging regions %d and %d at window size %d would produce no new interior windows\n', region1.label, region2.label, windowSize);
        jointRegion.interiorPixelCounts(windowIndex) = 0;
    end
    
end