% COMBINE_MEAN_COV compute the joint sample mean and covariance, given the
% sample means, covariances, and sampleCounts from two Gaussian data sets
%
%   [mu_hat, Sigma_hat, n_hat] = combine_mean_cov(mu1, Sigma1, n1, mu2,
%      Sigma2, n2);
%
%   mu1 is the sample mean of the first set of data
%
%   Sigma1 is the sample covariance of the first set of data
%
%   n1 is the number of samples in the first set of data
%
%   mu2 is the sample mean of the second set of data
%
%   Sigma2 is the sample covariance of the second set of data
%
%   n2 is the number of samples in the second set of data
%
%   mu_hat is the sample mean of the joint set of data
%
%   Sigma_hat is the sample covariance of the joint set of data
%
%   n_hat is the number of samples in the joint set of data
%
%Authors: Shankar Rao
%Contact: Shankar Rao <srrao@illinois.edu>
%
%(c) Copyright. University of Illinois at Urbana-Champaign.
%               University of California, Berkeley. 2008.
%
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of their author(s). The authors are not responsible
%for any potential property loss or damage caused directly or indirectly by
%the usage of the software.

function [mu_hat, Sigma_hat, n_hat] = combine_mean_cov(mu1, Sigma1, n1, mu2, Sigma2, n2);
if n1==0 && n2==0
    mu_hat = zeros(size(mu1));
    Sigma_hat = zeros(size(Sigma1));
    n_hat = 0;
elseif n1==0,
    mu_hat = mu2;
    Sigma_hat = Sigma2;
    n_hat = n2;
elseif n2==0,
    mu_hat = mu1;
    Sigma_hat = Sigma1;
    n_hat = n1;
else
    n_hat = n1+n2;
    alpha1 = n1/n_hat;
    alpha2 = n2/n_hat;
    mu_hat = alpha1*mu1+alpha2*mu2;
    Sigma1_nomean = Sigma1 + mu1*mu1';
    Sigma2_nomean = Sigma2 + mu2*mu2';
    Sigma_hat = alpha1*Sigma1_nomean + alpha2*Sigma2_nomean - mu_hat*mu_hat';
end