function [pixelFeatures, maxMean, bases, eigenvalues, sampleCenter] = construct_pixel_features(image, windowSizeArray, parameters)

if nargin==0,
    pixelFeatures.colorMetric = 'lab';
    pixelFeatures.normalization = 'std';
    pixelFeatures.applyGaussianFilter = true;
    pixelFeatures.GaussianVariance = 0.5;
    pixelFeatures.PCASamplingFactor = 1;
    pixelFeatures.PCADownsampleDimension = 8;
    pixelFeatures.centerPCA = true;
    pixelFeatures.recompute = false;
    pixelFeatures.save = true;
    pixelFeatures.filename = '';
    pixelFeatures.DEBUG = 2;
    if nargout >= 2,
        maxMean = {};
    end
    if nargout >= 3,
        bases = {};
    end
    return;
end

switch lower(parameters.normalization)
    case 'none'
        normalization = ones(3,1);
    case '133'
        normalization = [1;3;3];
        normalization = normalization / (prod(normalization)^(1/3));
    case 'std'
        normalization = [1; 7.7459 ; 4.7679];
        normalization = normalization / (prod(normalization)^(1/3));
end


maxMean = cell(length(windowSizeArray),1);
bases = cell(length(windowSizeArray),1);
eigenvalues = cell(length(windowSizeArray),1);

if parameters.centerPCA,
    sampleCenter = cell(length(windowSizeArray),1);
end
if ~exist(parameters.filename, 'file') || parameters.recompute || isempty(who('-file', parameters.filename, 'pixelFeatures'))
    % Construct and homogenize sample vectors
    [imageX, imageY, channelCount] = size(image);
    warning off
    if strcmp(parameters.colorMetric,'rgb')
        imageDouble = im2double(image);
        channelCount = 3;
    elseif strcmp(parameters.colorMetric,'lab')
        colorForm = makecform('srgb2lab');
        imageDouble = double(applycform(image,colorForm));
        for channelIndex = 1:channelCount,
            imageDouble(:,:,channelIndex) = normalization(channelIndex)*imageDouble(:,:,channelIndex);
        end
    elseif strcmp(parameters.colorMetric,'uvl')
        colorForm = makecform('srgb2xyz');
        imageDouble = applycform(image,colorForm);
        colorForm = makecform('xyz2uvl');
        imageDouble = applycform(imageDouble,colorForm);
        imageDouble = imageDouble(:,:,1:channelCount);
    end
    warning on
    
    windowSizeCount = length(windowSizeArray);
    pixelFeatures = cell(windowSizeCount,1);
    sampleCount = imageX*imageY;
    for windowIndex = 1:windowSizeCount
        
        windowSize = windowSizeArray(windowIndex);
        if parameters.DEBUG > 0
            fprintf('    Current Window Size = %d x %d\n', windowSize, windowSize);
        end
                halfWindowSize = floor(windowSize/2);
                        sampleDimension = windowSize*windowSize*channelCount;
        downsampleDimension = min(sampleDimension, parameters.PCADownsampleDimension);
        
        
        
        
        imageBlur = imageDouble;
        if parameters.applyGaussianFilter && halfWindowSize>0
            GaussianMask = torr_gauss_mask(halfWindowSize, halfWindowSize*parameters.GaussianVariance);
            for channelIndex =1:channelCount,
                imageBlur(:,:,channelIndex) = conv2(imageDouble(:,:,channelIndex), GaussianMask, 'same');
            end
        end

        
        
        % first, randomly sample windows from the image to compute the covariance
        % matrix
        varianceMatrix = zeros(sampleDimension);
        sampleCount = 0;
        if parameters.centerPCA,
            sampleCenter{windowIndex} = zeros(sampleDimension,1);
        end
        for xIndex = 1+halfWindowSize:imageX-halfWindowSize,
            for yIndex = 1+halfWindowSize:imageY-halfWindowSize,
                if rand < parameters.PCASamplingFactor

                    sampleWindow = imageBlur(xIndex-halfWindowSize:xIndex+halfWindowSize, ...
                        yIndex-halfWindowSize:yIndex+halfWindowSize,:);

                    currentSample = sampleWindow(:);
                    varianceMatrix = varianceMatrix + currentSample*currentSample';
                    if parameters.centerPCA,
                        sampleCenter{windowIndex} = sampleCenter{windowIndex} + currentSample;
                    end
                    sampleCount = sampleCount + 1;
                end
            end
        end
        
        varianceMatrix = varianceMatrix / sampleCount;
        if parameters.centerPCA,
            sampleCenter{windowIndex} = sampleCenter{windowIndex}  / sampleCount;
            varianceMatrix = varianceMatrix - sampleCenter{windowIndex}*sampleCenter{windowIndex}';
        end
        [U S] = svd(varianceMatrix,0);
        U = U(:, 1:downsampleDimension);

        bases{windowIndex} = U;        
        eigenvalues{windowIndex} = diag(S);
        
        % We store the pixel features in a 2-D array because they are significantly
        % more efficient than 3-D arrays in MATLAB
        pixelFeatures{windowIndex} = zeros(downsampleDimension, sampleCount);
        for yIndex = 1+halfWindowSize:imageY-halfWindowSize,
            for xIndex = 1+halfWindowSize:imageX-halfWindowSize,
                
                sampleWindow = imageDouble(xIndex-halfWindowSize:xIndex+halfWindowSize, ...
                    yIndex-halfWindowSize:yIndex+halfWindowSize,:);
                if parameters.applyGaussianFilter && halfWindowSize>0,
                    for channelIndex=1:channelCount
                        sampleWindow(:,:,channelIndex) = conv2(sampleWindow(:,:,channelIndex),GaussianMask ,'same');
                    end
                end
                currentSample = sampleWindow(:);
                if parameters.centerPCA,
                    currentSample = currentSample - sampleCenter{windowIndex};
                end
                
                pixelFeatures{windowIndex}(:,sub2ind([imageX imageY], xIndex, yIndex)) = U'*currentSample;

            end
        end
        maxMean{windowIndex} = U'*(255*ones(sampleDimension,1));
    end
    if parameters.save && ~isempty(parameters.filename),
        if parameters.centerPCA,
            save(parameters.filename,'pixelFeatures', 'maxMean', 'bases', 'eigenvalues', 'sampleCenter');
        else
            save(parameters.filename,'pixelFeatures', 'maxMean', 'bases', 'eigenvalues');
        end
    end
else
    if parameters.centerPCA,
        load(parameters.filename,'pixelFeatures', 'maxMean', 'bases', 'eigenvalues', 'sampleCenter');
    else
        load(parameters.filename,'pixelFeatures', 'maxMean', 'bases', 'eigenvalues');
    end
end