function regions  = construct_region_parameters(pixelFeatures, pixelLabels, windowSizeArray, useUniformCode)
% CONSTRUCT_REGION_PARAMETERS for each region in the image, compute the
% mean, covariance, number of interior pixels (pixels whose windows all lie
% in the same region as the pixel), total number of pixels, and the
% frequency counts of each difference chain code. These region parameters
% are computed for a number of different window sizes where appropriate
%
% regions  = construct_region_parameters(pixelFeatures, pixelLabels,
% windowSizeArray)
%
% pixelFeatures is a W x 1 cell array where the ith entry is a data matrix
% containing the features for each pixel according the window size index by
% W in windowSize array. W is the number of window sizes considered.
%
% pixelLabels is an integer array the size of the image whose ij-th element
% contains the label of the pixel at (i,j) in the image.
%
% windowSizeArray is a W x 1 list of window sizes considered. Each entry
% must be odd and the list should contain 1 and be strictly increasing.
%
% regions is an R x 1 structure array where the ith entry contains the
% parameters of the ith region. R is the number of regions in the image.
%
%Authors: Shankar Rao, Hossein Mobahi, Allen Yang
%Contact: Shankar Rao <srrao@illinois.edu>
%(c) Copyright. University of Illinois at Urbana-Champaign. 2009.
%               University of California, Berkeley. 2009.
%
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of their author(s). The authors are not responsible
%for any potential property loss or damage caused directly or indirectly by
%the usage of the software.



if nargin < 4
    useUniformCode = false;
end


imageSize = size(pixelLabels);
regionCount = max(max(pixelLabels));


regions = struct('label', cell(regionCount,1), ...
    'center', cell(regionCount,1), ...
    'covariance', cell(regionCount,1), ...
    'interiorPixelCounts', cell(regionCount, 1), ...
    'totalPixelCount', cell(regionCount, 1), ...
    'chainCodeCounts', cell(regionCount, 1));


windowSizeCount = length(windowSizeArray);



for regionIndex=1:regionCount,

        currentRegion = struct('label', regionIndex, ...
            'center', {cell(windowSizeCount,1)}, ...
            'covariance', {cell(windowSizeCount,1)}, ...
            'interiorPixelCounts', zeros(1,windowSizeCount), ...
            'totalPixelCount', 0, ...
            'chainCodeCounts', zeros(1,8));


    foundNondegenerate = false;
    for windowIndex = 1:windowSizeCount
        dimensionCount= size(pixelFeatures{windowIndex}, 1);
        windowSize = windowSizeArray(windowIndex);
        pixelIndices = find_interior_pixels(pixelLabels, regionIndex, windowSize);
        
        if ~isempty(pixelIndices)
            foundNondegenerate = true;
            [mu, Sigma, n] = construct_mean_cov(pixelFeatures{windowIndex}, pixelIndices);
            currentRegion.center{windowIndex} = mu;
            currentRegion.covariance{windowIndex} = Sigma;
            currentRegion.interiorPixelCounts(windowIndex) = n;

        else
            currentRegion.center{windowIndex} = zeros(dimensionCount,1);
            currentRegion.covariance{windowIndex} = zeros(dimensionCount);

        end
    end
    if ~foundNondegenerate 
        %warning(sprintf('Region %d contains no interior windows for any of the given window sizes', regionIndex));
    end
    currentRegion.totalPixelCount = sum(sum(pixelLabels==regionIndex));
    inThisRegion = pixelLabels == regionIndex;
    currentRegion.chainCodeCounts = adaptive_chain_code(inThisRegion, useUniformCode);
    regions(regionIndex) = currentRegion;
end



