function [superpixelLabels, regionCount] = obtain_superpixels(image, parameters)
% OBTAIN_SUPERPIXELS obtain an oversegmentation of an image that preserves
% strong edges. Currently there are two methods for doing so, one proposed
% by Mori et al. using normalized cut, and one proposed by Felzenszwalb and
% Huttenlocher using a fast graph-based approach.
%
% [superpixelLabels, regionCount] = obtain_superpixels(image, parameters)
%
% image is an MxNx3 image whose entries are unsigned integers.
%
% parameters is a struct contain the following parameters of the two super
% pixel method:
%
%  - save is true if the superpixel segmentation should be saved
%  to a file. Default is false.
%
%  - filename is the name of the file that the superpixel segmentation
%  should be saved to (or loaded from). It is an error if this is empty and
%  save is true.
%  Default is ''.
%
%  - recompute is true if the segmentation should be recomputed instead of
%  loaded from a file. If it is false and no file is specified (or the file
%  does not exist), then the segmentation will be recomputed anyway.
%  Default is true.
%
%  - N_sp is the desired number of superpixel in Mori's approach. Default
%  is 200.
%
%  superpixelLabels is an MxN array containing the label of each pixel in
%  the superpixel segmentation. If no arguments are provided to the
%  function then this is instead a structure containing all of the default
%  paramters.
%
%  regionCount is the number of superpixels in the superpixel segmentation.
%
%
%Authors: Shankar Rao
%Contact: Shankar Rao <srrao@illinois.edu>
%
%(c) Copyright. University of Illinois at Urbana-Champaign.
%               University of California, Berkeley. 2009.
%
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of their author(s). The authors are not responsible
%for any potential property loss or damage caused directly or indirectly by
%the usage of the software.
if nargin == 0,
    superpixelLabels.save = true;
    superpixelLabels.filename = '';
    superpixelLabels.recompute = false;
    % Parameters for Mori's superpixel segmentation
    superpixelLabels.N_sp=200;
    
    if nargout == 2,
        regionCount = 0;
    end
    return;
end

if nargin <2
    parameters = obtain_superpixels;
end

if ~exist(parameters.filename, 'file') || parameters.recompute || isempty(who('-file', parameters.filename, 'superpixelLabels'))
    if ~parameters.recompute && ~exist(parameters.filename, 'file'),
        fprintf('Warning: Superpixel file %s can not be found.\n', parameters.filename);
    end
    [imageX, imageY, imageDepth] = size(image);
    
    % Pre-processing step, compute superpixels
    imageDouble = im2double(image);
    
    % Number of eigenvectors.
    N_ev=40;
    
    % ncut parameters for superpixel computation
    diag_length = sqrt(imageX*imageX + imageY*imageY);
    par = imncut_sp;
    par.int=0;
    par.pb_ic=1;
    par.sig_pb_ic=0.05;
    par.sig_p=ceil(diag_length/50);
    par.verbose=0;
    par.nb_r=ceil(diag_length/60);
    par.rep = -0.005;  % stability?  or proximity?
    par.sample_rate=0.2;
    par.nv = N_ev;
    par.sp = parameters.N_sp;
    
    % Intervening contour using mfm-pb
    fprintf('running PB\n');
    [emag,ephase] = pbWrapper(imageDouble,par.pb_timing);
    emag = pbThicken(emag);
    par.pb_emag = emag;
    par.pb_ephase = ephase;
    clear emag ephase N_ev N_sp;
    
    fprintf('Ncutting...\n');
    superpixelLabels = imncut_sp(imageDouble,par);
    if parameters.save && ~isempty(parameters.filename),
        save(parameters.filename,'superpixelLabels');
    end
    clear par
    
else
    load(parameters.filename, 'superpixelLabels');
end

regionCount = max(max(superpixelLabels));

regionIndex2 = 0;
for regionIndex=1:regionCount
    pixelCoordinates = find(superpixelLabels==regionIndex);
    
    % Eliminate empty classes -- this may not be necessary now
    % Actually is necessary after my modification (Hossein)
    if ~isempty(pixelCoordinates)
        regionIndex2  = regionIndex2 + 1;
        superpixelLabels(pixelCoordinates) = regionIndex2;
    end
end