% TEXTURE_SEG
%
% A lossy coding based image segmentor. This version ignores the windows on
% the boundary between two regions when fitting a texture model to a given
% region. It also uses the adaptive chain code to represent the boundary between
% two regions.
%
%
%Authors: Shankar Rao, Hossein Mobahi, Allen Yang
%Contact: Shankar R. Rao <srrao@illinois.edu>
%
%(c) Copyright. University of Illinois, 2009.
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of the authors.

function [sampleLabels, syntheticImage, finalRegions, finalVicinityMaps, isGoodEpsilon]=texture_seg(image,varargin)



% set default arguments
currentDirectory = [];
DEBUG = 1;
maxWindowSize = 7;
initialSampleLabels = [];       % Set an unassigned value
channelCount = 3;

superpixel = obtain_superpixels;

features = construct_pixel_features;

epsilon = 100;

coding = coding_length_image;

segmentation.save = true;
segmentation.filename = '';
segmentation.mergeEdgelessRegions = true;
segmentation.edgeMethod = 'gradient';
segmentation.edgeGradientThreshold = 1;
segmentation.strongEdgeThreshold = 0.05;
segmentation.saveIntermediates = true;
segmentation.intermediatePrefix = 'seg';




% Parse the optional inputs.
if (mod(length(varargin), 2) ~= 0 )
    error(['Extra Parameters passed to the function ''' mfilename ''' must be passed in pairs.']);
end
parameterCount = length(varargin)/2;

for parameterIndex = 1:parameterCount,
    parameterName = varargin{parameterIndex*2 - 1};
    parameterValue = varargin{parameterIndex*2};
    switch lower(parameterName)
        case 'applygaussianfilter'
            features.applyGaussianFilter = parameterValue;
            if parameterValue~=true && parameterValue~=false
                error(['The parameter ''' parameterName ''' must be boolean.']);
            end
        case 'colormetric'
            features.colorMetric = lower(parameterValue);
            if ~strcmp(parameterValue,'rgb') && ~strcmp(parameterValue,'lab') && ...
                    ~strcmp(parameterValue,'uvl')
                error('colorMetric only supports ''RGB'', ''LAB'', and ''UVL''. ');
            end
        case 'codeboundary'
            coding.codeBoundary = parameterValue;
            if parameterValue~=0 && parameterValue~=1
                error(['The parameter ''' parameterName ''' is out of range']);
            end
        case 'chaincodemode'
            coding.chainCodeMode = parameterValue;
            if ~strcmpi(parameterValue, 'adaptive') && ~strcmpi(parameterValue, 'uniform'),
                error(['The parameter ''' parameterName ''' can only be ''adaptive'' or ''uniform''.']);
            end
        case 'channelcount'
            channelCount = parameterValue;  
        case 'debug'
            DEBUG = parameterValue;
            features.DEBUG = parameterValue;
        case 'downsampledimension'
            features.PCADownsampleDimension = parameterValue;
            if parameterValue<=1
                error(['The parameter ''' parameterName ''' is out of range']);
            end            
        case 'epsilon'
            epsilon = parameterValue;            
            % Parameters for F&H based super pixel segmentation
        case 'gaussianvariance'
            % The variance of the Gaussian smoothing kernel w.r.t the
            % window size.
            features.GaussianVariance = parameterValue;
            if parameterValue<=0
                error(['The parameter ''' parameterName ''' is out of range']);
            end            
        case 'intermediateprefix'
            segmentation.intermediatePrefix = parameterValue;
        case 'mergeedgelessregions'
            segmentation.mergeEdgelessRegions = parameterValue;
            if parameterValue~=true && parameterValue~=false,
                error(['The parameter ''' parameterName ''' is out of range']);
            end
        case 'n_sp'
            superpixel.N_sp = parameterValue;
        case 'overlappingwindows'
            coding.overlappingWindows = parameterValue;
            if parameterValue~=0 && parameterValue~=1
                error(['The parameter ''' parameterName ''' is out of range']);
            end
        case 'pcasamplingfactor'
            features.PCASamplingFactor = parameterValue;
            if parameterValue<=0 || parameterValue>=1
                error(['The parameter ''' parameterName ''' is out of range']);
            end     
        case 'readsamplelabels'
            readSampleLabels = parameterValue;
            if parameterValue~=0 && parameterValue~=1
                error(['The parameter ''' parameterName ''' is out of range']);
            end            
        case 'recomputefeatures'
            features.recompute = parameterValue;
            if parameterValue~=0 && parameterValue~=1
                error(['The parameter ''' parameterName ''' is out of range']);
            end
        case 'recomputesuperpixels'
            superpixel.recompute = parameterValue;
            if parameterValue~=0 && parameterValue~=1
                error(['The parameter ''' parameterName ''' is out of range']);
            end            
        case 'featuresfilename'
            features.filename = parameterValue;
        case 'savefeatures'
            features.save = parameterValue;
            if parameterValue~=0 && parameterValue~=1
                error(['The parameter ''' parameterName ''' is out of range']);
            end
        case 'saveintermediates'
            segmentation.saveIntermediates = parameterValue;
            if parameterValue~=0 && parameterValue~=1
                error(['The parameter ''' parameterName ''' is out of range']);
            end    
        case 'savesuperpixels'
            superpixel.save = parameterValue;
            if parameterValue~=0 && parameterValue~=1
                error(['The parameter ''' parameterName ''' is out of range']);
            end            
        case 'superpixelfilename'
            superpixel.filename = parameterValue;            
        case 'superpixelmethod'
            superpixel.method = parameterValue;
            if ~strcmpi(parameterValue, 'fh') && ~strcmpi(parameterValue, 'mori'),
                error(['The parameter ''' parameterName ''' can only be ''FH'' or ''Mori''.']);
            end

        case 'windowsize'
            maxWindowSize = parameterValue;
            if any(parameterValue<1) || any(rem(parameterValue,2)==0)
                error('windowSize must be an odd natural number.');
            end   
        case 'currentdirectory'
            currentDirectory = parameterValue;
        otherwise
            error([parameterName ' is an invalid parameter.']);
    end
end

if (features.save==true) && (isempty(features.filename))
    features.save = false;
end
if (superpixel.save==true) && (isempty(superpixel.filename))
    superpixel.save = false;
end

if maxWindowSize<1 || any(rem(maxWindowSize,2)~=1),
    error('The parameter windowSize must be an odd natural number.');
end


windowSizeArray = maxWindowSize:-2:1;

if isempty(currentDirectory)
    currentDirectory = cd;
end
% set paths to helper functions
if ~exist(superpixel.filename, 'file') || superpixel.recompute || isempty(who('-file', superpixel.filename, 'superpixelLabels'))
    addpath([currentDirectory '/superpixels']);
    addpath([currentDirectory '/superpixels/yu_imncut']);
    addpath([currentDirectory '/superpixels/lib/matlab']);
end

[imageX, imageY, imageDepth] = size(image);
if imageDepth~=3
    error('The input image must have 3 channels.');
end


if DEBUG >0
    disp('****** Step I: Compute Super Pixel segmentation ******');
end

% Obtain an initial segmentation
% Pre-processing step, compute superpixels
if isempty(initialSampleLabels)
    pixelLabels = obtain_superpixels(image, superpixel);
else
    pixelLabels = initialSampleLabels;
    clear initialSampleLabels
end



if DEBUG >0,
    disp('****** Step II: Construct features for each window size  ******');
end

[pixelFeatures coding.maxMeans U] = construct_pixel_features(image, windowSizeArray, features);

if segmentation.mergeEdgelessRegions && strcmp(segmentation.edgeMethod, 'gradient'),
    edgeImage = construct_edge_image(image, max(windowSizeArray), false, features);
else
    edgeImage = [];
end


if DEBUG >0,
    disp('****** Step III: Construct means, covariances, chain codes for each region  ******');
end

useUniformCode = strcmp(coding.chainCodeMode, 'uniform');

regions = construct_region_parameters(pixelFeatures, pixelLabels, windowSizeArray, useUniformCode);
    

% Construct the graph that models the adjacency relationships between
% regions
if DEBUG >0,
    disp('****** Step IV: Construct vicinity graph  ******');
end
 
vicinityMap = vicinity_map_fast(pixelLabels);         

% Segmentation
if DEBUG > 0
    disp('****** Step V: Segmentation ******');
end
epsilonCount = length(epsilon);

sampleLabels = zeros(imageX, imageY, epsilonCount);
syntheticImage = zeros(imageX, imageY, imageDepth, epsilonCount, 'uint8');
finalRegions = cell(epsilonCount,1);
finalVicinityMaps = cell(epsilonCount,1);
for epsilonIndex = 1:epsilonCount
    currentEpsilon = epsilon(epsilonIndex);
    if DEBUG > 0
        fprintf('  epsilon(%d of %d) = %f\n', epsilonIndex, epsilonCount, currentEpsilon);
    end
    intermediateFilename = [segmentation.intermediatePrefix '-' num2str(currentEpsilon), '.mat'];
    if segmentation.saveIntermediates && exist(intermediateFilename, 'file'),
        if DEBUG>0,
            fprintf('    Already exists ... loading from file\n');
        end
        load(intermediateFilename, 'tempLabels', 'currentRegions', 'currentVicinityMap');
    else
        [currentLabels, ...
            currentRegions, ...
            currentVicinityMap] = texture_seg_single_epsilon_decreasing_windows( ...
            pixelFeatures, pixelLabels, currentEpsilon, regions, vicinityMap, ...
            windowSizeArray, coding, DEBUG, image);
       
        
        % Create sampleLabels
        if DEBUG >0
            disp('****** Step V: Post-processing ******');
        end
        
        if segmentation.mergeEdgelessRegions,
            [currentRegions, currentLabels, currentVicinityMap] = ...
                merge_edgeless_regions(currentRegions, currentLabels, currentVicinityMap, image, ...
                segmentation, windowSizeArray, pixelFeatures, edgeImage,DEBUG);
        end
        
        
        % remove empty labels
        tempLabels = zeros(size(currentLabels));
        uniqueLabels = unique(currentLabels);
        currentRegions = currentRegions(uniqueLabels);
        currentVicinityMap = currentVicinityMap(uniqueLabels);
        for regionIndex = 1:length(uniqueLabels);
            tempLabels(currentLabels == uniqueLabels(regionIndex)) = regionIndex;
            currentRegions(regionIndex).label = regionIndex;
            for neighborIndex = 1:length(currentVicinityMap{regionIndex}),
                currentVicinityMap{regionIndex}(neighborIndex) = ...
                    find(uniqueLabels == currentVicinityMap{regionIndex}(neighborIndex));
            end
        end
        if segmentation.saveIntermediates,
            save(intermediateFilename, 'tempLabels', 'currentRegions', 'currentVicinityMap');
        end
    end
    sampleLabels(:,:,epsilonIndex) = tempLabels;
    finalRegions{epsilonIndex} = currentRegions;
    finalVicinityMaps{epsilonIndex} = currentVicinityMap;
    
    % Update syntheticImage
    if nargout>=2
        syntheticImage(:,:,:,epsilonIndex) = generate_synthetic_image_segments(image,sampleLabels(:,:,epsilonIndex));
    end
    
    if DEBUG>1
        h=figure;
        set(h,'Name',sprintf('Window Size = %d', maxWindowSize));
        imshow(syntheticImage(:,:,:,epsilonIndex));
        title(sprintf('epsilon = %f, window size = %d x %d, pca dim = %d', epsilon(epsilonIndex), maxWindowSize, maxWindowSize, features.PCADownsampleDimension));
        tempImage = segmented_image(image, sampleLabels(:,:,epsilonIndex));

        h=figure;
        set(h,'Name',sprintf('Window Size = %d', maxWindowSize));
        imshow(tempImage);
        title(sprintf('epsilon = %f, window size = %d x %d, pca dim = %d', epsilon(epsilonIndex), maxWindowSize, maxWindowSize, features.PCADownsampleDimension));
    end
end

    

    
