% TEXTURE_SEG_SINGLE_EPSILON
%
% Run the texture segmentation algorithm for a single choice of noise
% tolerance epsilon. The superpixel initialization and the mean and
% covariance of each region distribution does not have to be recomputed
% each time.
%
% function [sampleLabels, syntheticImage, regions] = texture_seg_single_epsilon(...
%     pixelFeatures, pixelLabels, epsilon,  regions, vicinityMap, ...
%     windowSizeArray, parameters, DEBUG, img)
%
% pixelFeatures is an D x N matrix containing the feature for each pixel in
% the image, indexed linearly. D is the number of features per pixel (the
% dimension projected onto by PCA), and N is the number of pixels in the
% image.
%
% pixelLabels is a matrix the ize of the image whose (i,j) entry contains
% the label of the corresponding pixel in the superpixel segmentation.
%
% epsilon is the specified noise tolerance
%
% regions is an R x 1 structure array containing the parameters of each region.
%  Each entry of the array contains the fields 'label', 'center', 'covariance',
%  'interiorPixelCounts', 'totalPixelCount', and 'chainCodeCounts'. R is the
%  number of regions in the superpixel segmentation.
%
% vicinityMap is a (1 x r) cell array where the ith entry contains the
% labels of regions adjacent to the ith region.
%
% windowSizeArray is the set of windows sizes around each pixel to be used.
% This must consist of odd integers, be strictly increasing, and contain
% the number 1 (because no window of size 1 can be degenerate).
%
% parameters is a structure containing the coding parameters. It includes
% the fields 'useChainCode', 'overlappingWindows', and 'chainCodeBitCounts'
%
% DEBUG is the debug output level. 0 = no output, 1 = text only, 2 =
% graphics as well, 3 = pausing
%
% img is the original image, used for debug plotting
%
% sampleLabels is a matrix the size of the image whose (i,j) entry contains
% the label of the corresponding pixel in the final segmentation.
%
% syntheticImage is an image with each distinct region colored randomly.
%
%Authors: Shankar Rao, Hossein Mobahi, Allen Yang
%Contact: Shankar R. Rao <srrao@illinois.edu>
%
%(c) Copyright. University of Illinois, 2008.
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of the authors.

function [sampleLabels, regions, vicinityMap] = texture_seg_single_epsilon_decreasing_windows(...
    pixelFeatures, pixelLabels, epsilon,  regions, vicinityMap, ...
    windowSizeArray, parameters, DEBUG, img)

[imageX, imageY] = size(pixelLabels);

initialRegionCount = length(regions);
regionCount = initialRegionCount;
windowSizeCount = length(windowSizeArray);
regionMDL = zeros(windowSizeCount, regionCount);

epsilon2 = (windowSizeArray*epsilon./max(windowSizeArray)).^2;

useUniformCode = strcmp(parameters.chainCodeMode, 'uniform');



pixelCountThreshold = 5;
for segmentationRound = 1:2,
    if segmentationRound==1,
        maxWindowIndex = 1;
    else
        maxWindowIndex = windowSizeCount;
    end
    
    for regionIndex=1:initialRegionCount
        if isempty(vicinityMap{regionIndex}),
            continue;
        end
        for windowIndex = 1:maxWindowIndex,
            if  regions(regionIndex).interiorPixelCounts(windowIndex)==0
                regionMDL(windowIndex, regionIndex) = 0;
            else
                regionMDL(windowIndex, regionIndex) = coding_length_image( ...
                    regions(regionIndex), epsilon2(windowIndex), [imageX imageY], ...
                    windowIndex, windowSizeArray(windowIndex), parameters);
            end
        end
    end
    
    % compute the change in coding length for merging each pair of regions
    if DEBUG > 0
        if segmentationRound == 1,
            disp('    Computing change in coding length for each pair of initial regions...');
        else
            disp('    Recomputing change in coding length for smaller window sizes ...');
        end
    end
    
    mergeCount = 0;
    mergeMDLDifference = [];
    mergeRecord = [];
    for regionIndex=1:initialRegionCount,
        if isempty(vicinityMap{regionIndex}),
            continue;
        end
        currentRegion = regions(regionIndex);
        for vicinityIndex=1:length(vicinityMap{regionIndex})
            vicinityRegion = regions(vicinityMap{regionIndex}(vicinityIndex));
            if vicinityRegion.label<regionIndex
                % To avoid computing merging twice, enforce in mergeRecord, 1<2
                continue;
            end
         
            jointRegion = combine_joint_region_parameters( ...
                currentRegion, vicinityRegion, pixelLabels, pixelFeatures, ...
                1:maxWindowIndex, windowSizeArray(1:maxWindowIndex), useUniformCode);
            mergeCount = mergeCount + 1;
            mergeRecord(mergeCount,1:2) = [regionIndex vicinityRegion.label];
            mergeMDLDifference(1:windowSizeCount, mergeCount) = -Inf;
            for windowIndex = 1:maxWindowIndex,
                pixelCountDifference = jointRegion.interiorPixelCounts(windowIndex) ...
                    - vicinityRegion.interiorPixelCounts(windowIndex) ...
                    - currentRegion.interiorPixelCounts(windowIndex);
                if currentRegion.interiorPixelCounts(windowIndex)~=0 && ...
                        vicinityRegion.interiorPixelCounts(windowIndex)~=0 && ...
                        (windowIndex == windowSizeCount || pixelCountDifference > pixelCountThreshold) && ...
                        (windowIndex == 1 || ...
                        currentRegion.interiorPixelCounts(windowIndex-1)==0 || ...
                        vicinityRegion.interiorPixelCounts(windowIndex-1)==0),
                    
                    jointMDL = coding_length_image(jointRegion, epsilon2(windowIndex), ...
                        [imageX imageY], windowIndex, windowSizeArray(windowIndex), parameters);
                    mergeMDLDifference(windowIndex, mergeCount) = ...
                        regionMDL(windowIndex, vicinityRegion.label) + ...
                        regionMDL(windowIndex, regionIndex) - jointMDL;
                end
            end
        end
        
    end
    
    while (regionCount>1)
        
        for windowIndex = 1:maxWindowIndex,
            [maxMergeMDLDifference, maxMDLIndex ]= max(mergeMDLDifference(windowIndex,:));
            if maxMergeMDLDifference > 0,
                break;
            end
        end
        if segmentationRound==2 && maxMergeMDLDifference <=0,
            for windowIndex = maxWindowIndex:-1:2,
                [maxMergeMDLDifference, maxMDLIndex ]= max(mergeMDLDifference(windowIndex,:));
                if maxMergeMDLDifference > -Inf,
                    break;
                end
            end
        end
        if ~(maxMergeMDLDifference > -Inf) || (segmentationRound==1 && maxMergeMDLDifference <= 0),
            break;
        end
        if maxMergeMDLDifference <= 0 && DEBUG>1,
            fprintf('Warning: Merging regions that increases the coding length\n');
        end
        windowSize = windowSizeArray(windowIndex);
        
        % Merge mergeRecord(1,1) to mergeRecord(1,2)
        mergedIndex1 = mergeRecord(maxMDLIndex,1);
        mergedIndex2 = mergeRecord(maxMDLIndex,2);
        
        if DEBUG>1
            fprintf('Region Count:%d Merging groups %d and %d using %d x %d windows, reducing coding length by %f\n', ...
                regionCount, mergedIndex1, mergedIndex2, windowSize, windowSize, maxMergeMDLDifference);
            fprintf('  Region 1: %d total pixels %s interior pixels %f coding length\n', ...
                regions(mergedIndex1).totalPixelCount, ...
                sprintf('%d ', regions(mergedIndex1).interiorPixelCounts), regionMDL(windowIndex,mergedIndex1));
            fprintf('    Chain Code Frequencies: [%s]\n', sprintf('%d ', sum(regions(mergedIndex1).chainCodeCounts,1)));
            fprintf('  Region 2: %d total pixels %s interior pixels %f coding length\n', ...
                regions(mergedIndex2).totalPixelCount, ...
                sprintf('%d ', regions(mergedIndex2).interiorPixelCounts), regionMDL(windowIndex,mergedIndex2));
            fprintf('    Chain Code Frequencies: [%s]\n', sprintf('%d ', sum(regions(mergedIndex2).chainCodeCounts,1)));
            testImage = zeros(imageX, imageY, 3, 'uint8');
            [xIndices, yIndices] = find(pixelLabels == mergedIndex1 | pixelLabels == mergedIndex2);
            for i=1:length(xIndices)
                if isa(img, 'uint8')
                    testImage(xIndices(i),yIndices(i), :) = img(xIndices(i),yIndices(i),:);                    
                else
                    testImage(xIndices(i),yIndices(i), :) = double(img(xIndices(i),yIndices(i),:))/max(max(max(testImage)));
                end
                
            end
            testImage = segmented_image(testImage, double(pixelLabels==mergedIndex1));
            testImage = segmented_image(testImage, double(pixelLabels==mergedIndex2));

            
        end
        
        
        % Update region parameters
        mergedRegion = combine_joint_region_parameters( ...
            regions(mergedIndex1), regions(mergedIndex2), pixelLabels, ...
            pixelFeatures, 1:windowSizeCount, windowSizeArray, useUniformCode);

        regions(mergedIndex1) = mergedRegion;
        
        
        % Update MDL
        for windowIndex = 1:maxWindowIndex,
            if  mergedRegion.interiorPixelCounts(windowIndex)==0
                regionMDL(windowIndex, mergedIndex1) = 0;
            else
                regionMDL(windowIndex, mergedIndex1) = coding_length_image( ...
                    mergedRegion, epsilon2(windowIndex), [imageX imageY], ...
                    windowIndex, windowSizeArray(windowIndex), parameters);
            end
        end
        regionMDL(:, mergedIndex2) = 0;
        
        % Update pixel labels
        pixelLabels(pixelLabels == mergedIndex2) = mergedIndex1;
        if DEBUG>1
            fprintf('  New Region: %d total pixels ( %s) interior pixels %f coding length\n', mergedRegion.totalPixelCount, ...
                sprintf('%d ', mergedRegion.interiorPixelCounts), regionMDL(find(windowSizeArray==windowSize),mergedIndex1));
            fprintf('    Chain Code Frequencies: [%s]\n', sprintf('%d ', sum(mergedRegion.chainCodeCounts,1)));
            if DEBUG>2
                pause
            end
        end
        % Update vicinityMap
        for vicinityIndex=1:length(vicinityMap{mergedIndex2})
            % Change all vicinityMap indices that pointing at 2 toward 1
            neighborIndex = vicinityMap{mergedIndex2}(vicinityIndex);
            vicinityMap{neighborIndex} = union(vicinityMap{neighborIndex},mergedIndex1);
            vicinityMap{neighborIndex} = setdiff(vicinityMap{neighborIndex},mergedIndex2);
        end
        vicinityMap{mergedIndex1} = setdiff(union(vicinityMap{mergedIndex1}, ...
            vicinityMap{mergedIndex2}), [mergedIndex1 mergedIndex2]);
        vicinityMap{mergedIndex2} = [];
        
        % Update mergeMDLDifference
        mergeRecord = mergeRecord([1:maxMDLIndex-1, maxMDLIndex+1:end],:);
        mergeMDLDifference = mergeMDLDifference(:, [1:maxMDLIndex-1, maxMDLIndex+1:end]);
        mergeCount = mergeCount - 1;
        
        remainingIndices = [];
        for mergeIndex=1:mergeCount
            if (mergeRecord(mergeIndex,1)~=mergedIndex1) && ...
                    (mergeRecord(mergeIndex,1)~=mergedIndex2) && ...
                    (mergeRecord(mergeIndex,2)~=mergedIndex1) && ...
                    (mergeRecord(mergeIndex,2)~=mergedIndex2),
                % Keep this index item
                remainingIndices = [remainingIndices mergeIndex];
            end
        end

        mergeRecord = mergeRecord(remainingIndices,:);
        mergeMDLDifference = mergeMDLDifference(:, remainingIndices);
        mergeCount = length(remainingIndices);
        
        % The just deleted mergeMDL need to be recomputed
        currentVicinityMap = vicinityMap{mergedIndex1};
        for mergeIndex=1:length(currentVicinityMap)
            vicinityRegion = regions(currentVicinityMap(mergeIndex));

            % Joint mean and covariance of potential merged region
            jointRegion = combine_joint_region_parameters( ...
                mergedRegion, vicinityRegion, pixelLabels, pixelFeatures, ...
                1:maxWindowIndex, windowSizeArray(1:maxWindowIndex),useUniformCode);
            mergeCount = mergeCount + 1;
            mergeRecord(mergeCount,1)=min(mergedIndex1,vicinityRegion.label);
            mergeRecord(mergeCount,2)=max(mergedIndex1,vicinityRegion.label);
            
            mergeMDLDifference(1:windowSizeCount, mergeCount) = -Inf;
            for windowIndex = 1:maxWindowIndex,
                pixelCountDifference = jointRegion.interiorPixelCounts(windowIndex) ...
                    - vicinityRegion.interiorPixelCounts(windowIndex) ...
                    - mergedRegion.interiorPixelCounts(windowIndex);
                if mergedRegion.interiorPixelCounts(windowIndex)~=0 ...
                        && vicinityRegion.interiorPixelCounts(windowIndex)~=0 ...
                        && (windowIndex == windowSizeCount || pixelCountDifference > pixelCountThreshold)  ...
                        && (windowIndex == 1 ...
                        || mergedRegion.interiorPixelCounts(windowIndex-1)==0 ...
                        || vicinityRegion.interiorPixelCounts(windowIndex-1)==0),
                    jointMDL = coding_length_image(jointRegion, epsilon2(windowIndex), ...
                        [imageX imageY], windowIndex, windowSizeArray(windowIndex), parameters);
                    mergeMDLDifference(windowIndex, mergeCount) = regionMDL(windowIndex,mergedIndex1) + ...
                        regionMDL(windowIndex,vicinityRegion.label) - jointMDL;

                end
            end
        end
        regionCount = regionCount - 1;
    end
    
end


sampleLabels = pixelLabels;


