function codingLength = total_image_coding_length(regions, epsilon, imageSize, windowIndex, windowSize, parameters)
% TOTAL_IMAGE_CODING_LENGTH compute the total coding length of a particular
% segmentation of an image, subject to distortion epsilon^2
%
% codingLength = total_image_coding_length(regions, epsilon, imageSize, windowSize, parameters)
%
% regions is an R x 1 structure array containing the parameters of each region.
%  Each entry of the array contains the fields 'label', 'center', 'covariance', 
%  'interiorPixelCounts', 'totalPixelCount', and 'chainCodeCounts'. R is the 
%  number of regions in the segmentation.
%
%   epsilon2 is the square of the noise tolerance
%
%   imageSize is the dimensions of the image. only used if useChainCode is
%   false
%
%   windowSize is the number pixel along one edge of a window. only used if
%   overlappingWindows is true
%
%   parameters is a structure containing the coding parameters. It includes
%   the fields 'useChainCode', 'overlappingWindows', and 'chainCodeBitCounts'
%
%   - useChainCode is true we are using the chain code to model the
%   boundary of the image and false if we use the huffman code to model
%   group membership
%
%   - overlappingWindows is true if we model the windows associated to every
%   pixel in the region, or false if model only enough windows to represent 
%   the region in the image.
%
%   - chainCodeBitCounts is a (1 x 8) matrix containing the apriori frequency
%   of each chain code. Only used if useChainCode is true
%
%   jointRegion is a struct containing the parameters of the joint region.
%
%Authors: Shankar Rao
%Contact: Shankar Rao <srrao@illinois.edu>
%
%(c) Copyright. University of Illinois at Urbana-Champaign.
%               University of California, Berkeley. 2009.
%
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of their author(s). The authors are not responsible
%for any potential property loss or damage caused directly or indirectly by
%the usage of the software.

if nargin == 0
    codingLength = coding_length_image;
    return;
end

regionCount = length(regions);

epsilon2 = epsilon^2;
codingLength = 0;
for regionIndex = 1:regionCount
    codingLength = codingLength + coding_length_image(...
        regions(regionIndex), epsilon2, imageSize, windowIndex, windowSize, parameters);
end
